#include "gpsloghandler.h"
#include <stdlib.h>

using namespace std;

GpsLogHandler::GpsLogHandler(int recentTerm = 1, int recentLogNum = 600, int obsolateTerm = 15) {
	_stopped = true;
	_alive = false;
	_debug_print = false;
	_recentTerm = recentTerm;
	_recentLogNum = recentLogNum;
    _recentLog = new Coordinate[_recentLogNum +10];
    _recentNext = 0;
    _recentLogFull = false;
    _recentPtr = 0;

    _obsolateTerm = obsolateTerm;
    _obsolateIt = _obsolateLog.begin();
}

GpsLogHandler::~GpsLogHandler() {
	delete [] _recentLog;
	pthread_mutex_destroy(&_mutex_id);
}

void GpsLogHandler::setLogFile(QString logFile) {
	_gpsLogFile.setName(logFile);
}

void GpsLogHandler::setDevFile(QString devFile) {
	_gpsDevFile.setName(devFile);
}

Coordinate GpsLogHandler::getNowCoordinate() {
	pthread_mutex_lock(&_mutex_id);
	Coordinate tempCoordinate = _coordinate;
	pthread_mutex_unlock(&_mutex_id);
    return tempCoordinate;
}

void GpsLogHandler::start() {
	system((QString("cardctl resume;stty 4800 < ") + _gpsDevFile.name()).latin1());

	pthread_create(&_thread_id, NULL, thread_func, (void*)this);
	pthread_mutex_init(&_mutex_id, NULL);
}

void GpsLogHandler::run() {
    _stopped = true;
	// GPSǥХץ
	if (_gpsDevFile.exists()) {
		if (!_gpsDevFile.open(IO_ReadOnly)) {
			// GPS Device Open Error
			return;
		}
	} else {
		// GPS Device Not Exist Error
		return;
	}
	QTextStream devIn(&_gpsDevFile);    	

	// ե륪ץ
	if (_gpsLogFile.exists()) {
		if(!_gpsLogFile.open(IO_WriteOnly | IO_Append)) {
			// File Open Error.
			_gpsDevFile.close();
			return;
		}
	} else {
		if(!_gpsLogFile.open(IO_WriteOnly)) {
			// File Open Error.
			_gpsDevFile.close();
			return;
		}
	}
	QTextStream logOut(&_gpsLogFile);
    _stopped = false;
    
	// ɤ߹ߥ롼
    int recentCount = 0;
    int obsolateCount = 0;
	while(!(_stopped || devIn.eof())) {
		Radian rad;
		QString str = devIn.readLine();
		if (str.length() == 0)
		  continue;

		if (_debug_print)
			cout << "GPS> " << str << endl;

		// 񤭹
		logOut << str << "\n";

		if (str.left(6) == "$GPGGA") {
		  QStringList qslTemp = QStringList::split(",",str);

		  if (qslTemp[6] == "1") {
			pthread_mutex_lock(&_mutex_id);
			rad.parse(qslTemp[3] + qslTemp[2]);
		    _coordinate.setRadNS(rad);
		    rad.parse(qslTemp[5] + qslTemp[4]);
		    _coordinate.setRadEW(rad);
			QStringList qslHight = QStringList::split(".",qslTemp[9]);
			int intHight = qslHight[0].toInt() * 100;
			if (qslHight.count() >= 2)
				intHight += qslHight[1].toInt();
            _coordinate.setIntHight(intHight);
		    _coordinate.convWGS84ToTokyo();
		    _alive = true;
		    pthread_mutex_unlock(&_mutex_id);

		    if (_debug_print) {
		    	cout << "NS: " << qslTemp[3] + qslTemp[2] << " : " << _coordinate.getRadNS().toNS() << endl;
		    	cout << "EW: " << qslTemp[5] + qslTemp[4] << " : " << _coordinate.getRadEW().toEW() << endl;
		    	cout << "hight: " << qslTemp[9] << " : " << _coordinate.getIntHight() << endl;
		    }

		    if (recentCount == 0) {
		    	_recentLog[_recentNext++] = _coordinate;
		    	if (_recentNext >= _recentLogNum) {
		    		_recentNext = 0;
		    		_recentLogFull = true;
		    	}
		    }
		    if (++recentCount >= _recentTerm)
		    	recentCount = 0;
		    
		    if (obsolateCount == 0)
		    	_obsolateLog.push_back(_coordinate);

		    if (++obsolateCount >= _obsolateTerm)
		    	obsolateCount = 0;
		    
		  } else {
			  _alive = false;			  
		  }
		} else {
			// $GPGGAʳNMEA
		}
	}

	_gpsDevFile.close();
	_gpsLogFile.close();
	_stopped = true;
	_alive = false;
	cout << "Thread end." << endl;
}

void GpsLogHandler::changeDebugPrint() {
    if (_debug_print)
    	_debug_print = false;
    else
    	_debug_print = true;
}

void GpsLogHandler::stop() {
	cout << "Thread stopping." << endl;
	_stopped = true;
	pthread_join(_thread_id, NULL);

	system("cardctl suspend");
	cout << "Thread stopped." << endl;
}

bool GpsLogHandler::isStopped() {
	return _stopped;
}

void GpsLogHandler::setRecentLogFirst() {
	if (_recentLogFull) {
		_recentPtr = (_recentNext + 10) % _recentLogNum;
	} else {
		_recentPtr = _recentNext + 10 - _recentLogNum;
		if (_recentPtr < 0)
			_recentPtr = 0;
	}
}

bool GpsLogHandler::endRecentLog() {
	if (_recentPtr == _recentNext)
		return true;
	else
		return false;
}

Coordinate GpsLogHandler::getNextRecentLog() {
	int rtnIdx = _recentPtr++;
	if (_recentPtr >= _recentLogNum)
		_recentPtr = 0;
	return _recentLog[rtnIdx];
}

void GpsLogHandler::setObsolateLogFirst() {
	_obsolateIt = _obsolateLog.begin();
}

bool GpsLogHandler::endObsolateLog() {
	if (_obsolateIt == _obsolateLog.end())
		return true;
	else
		return false;
}

Coordinate GpsLogHandler::getNextObsolateLog() {
	return *(_obsolateIt++);
}

void* thread_func(void *pGpsLogHandler) {
  GpsLogHandler *myGpsLogHandler = (GpsLogHandler *)pGpsLogHandler;
  myGpsLogHandler->run();
}
